## 配置文件
可以通过--config 指定配置文件，如果未明确指定默认将寻找`alita.config.js`文件，如果未找到，将使用系统默认配置。

一般配置文件格式如下：

```javascript
module.exports = { 
    name:'',
    appid:'',
    
    isFileIgnore: (filepath) => false,
    subDir: '/',
    packageJSONPath: './package.json',
    dependencies: []
    
    
    /**
     *  dependenciesMap, extCompLibs 自Alita@1.3.0 版本之后，已经弃用，将在1.4.0之后彻底移除，请使用dependencies配置项
     */
    dependenciesMap: {
    },
    extCompLibs: [
    ]
}
```

### 字段说明
下面我们对每一个字段的含义分别说明

#### name
生成的微信小程序项目名
#### appid
调试小程序的appid，由微信申请而来

#### isFileIgnore
函数，参数是文件路径，如果返回`true`，文件将会被忽略
**注意** node_modules， .git， .expo等目录，不管这里配置与否转化引擎都会直接忽略

#### subDir
用来生成小程序分包，详见[小程序分包.md](./小程序分包.md)

#### packageJSONPath
指定React Native的package.json 文件，一般不需要指定


#### dependencies
由于小程序的[npm使用](https://developers.weixin.qq.com/miniprogram/dev/devtools/npm.html?search-key=npm)和React Native不同。
现阶段，Alita不会字段去处理`node_modules`目录，所有需要对RN的第三方依赖特别处理。

参考[npm包说明](./npm包说明.md)

参考[第三方组件扩展](./第三方组件库扩展.md)

总结一下，假如有`xxx`包

1. 若`xxx` 在微信小程序平台可以直接使用，不用处理

2. 若`xxx` 在微信小程序平台有对应的`xxx-wx`包，需要配置：
   
   ```javascript
   module.exports = {
       dependencies: [
           {
               name: 'xxx',
               wxName: 'xxx-wx',
           }
       ]
   }
   ```
   这样所有的RN文件：
   
   ```javascript
   import {a, b} from 'xxx'
   ```
   在小程序平台将会被处理为：
   
   ```javascript
    import {a, b} from 'xxx-wx'
   ```
   
3. 若`xxx` 因为[平台判断](./平台判断.md)等原因，只会在RN平台使用，需要配置：
   ```javascript
    module.exports = {
      dependencies: [
          {
              name: 'xxx',
              wxName: false,
          }
      ]
    }
    ```
    
4. 若`xxx` 是第三方组件， 且对外导出XXX组件，那么需要：
        
    ```javascript
    module.exports = {
      dependencies: [
          {
              name: 'xxx',
              wxName: 'xxx-wx', // <--- 对齐为 xxx-wx
           
              compLists: [
                  {
                      name: 'XXX',
                      path: '/index',
                  }
              ]
          }
      ]
    }
    ``` 
    这样所有的RN文件：
       
    ```javascript
    import XXX from 'xxx'
    ```
    在小程序平台将会被处理为：
   
    ```javascript
     import XXX from 'xxx-wx'
    ```
    且小程序定义组件引用的json文件，将根据`path: '/index'`， 生成如下json文件
    
    ```json
    {
    	"component": true,
    	"usingComponents": {
    		"XXX": "xxx-wx/index"
    	},
    	"componentGenerics": {}
    }
    ```
    
**需要特别强调的是：上面的处理过程，特别依赖`package.json` 文件的`dependencies` 字段，需要保证`dependencies` 字段的准确性，并把其他无关依赖
放置在`devDependencies`** 
   

#### dependenciesMap
弃用，使用dependencies代替

#### extCompLibs
弃用，使用dependencies代替
 